/*
KeePass for BlackBerry
 Copyright 2007 Fairview 5 Engineering,LLC<george.joseph@fairview5.com>

 This program is free software:you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation,either version 3 of the License,or
 (at your option)any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY;without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.If not,see<http://www.gnu.org/licenses/>.
 */

package com.fairview5.keepassbb;

import net.rim.device.api.synchronization.SyncObject;
import net.rim.device.api.system.CodeSigningKey;
import net.rim.device.api.system.DeviceInfo;
import net.rim.device.api.system.PersistentObject;
import net.rim.device.api.system.PersistentStore;
import net.rim.device.api.util.Persistable;

public class DatabaseObject implements SyncObject, Persistable {
  public static final byte FIELD_TYPE_TZ = 0x00;
  public static final byte FIELD_TYPE_DATA = 0x01;
  public static final long STORE_UID = 0xe41afdd75e5a954cL;

  private int uid;
  int dbLength;
  int dbFieldType;
  public byte[] db;

  DatabaseObject(int uid) {
    super();
    this.uid = uid;
  }

  DatabaseObject() {
    super();
    loadFromPersistentStore();
  }

  DatabaseObject(byte[] db) {
    super();
    this.db = db;
    this.dbLength = db.length;
    this.dbFieldType = FIELD_TYPE_DATA;
  }

  DatabaseObject(int uid, byte[] db) {
    this(db);
    this.uid = uid;
  }

  public int getUID() {
    return uid;
  }

  public String toString() {
    return ("DatabaseObject: " + uid);
  }


  public void loadFromPersistentStore()  {
    try {
      PersistentObject store = PersistentStore.getPersistentObject(STORE_UID);
      CodeSigningKey csk = CodeSigningKey.get(this);
      if (csk == null) {
        if (!DeviceInfo.isSimulator())
          throw new RuntimeException("Unable to retrieve signing key");
      }

      DatabaseObject sdb = (DatabaseObject) store.getContents();
      if (sdb == null || sdb.db == null)
        throw new RuntimeException("No database found. You should create or import one");
      
      /*
      ControlledAccess ca = store.getControlledAccess();
      boolean valid = ca.checkKeys(csk, csk);
      if (!valid) {
        if (!DeviceInfo.isSimulator())
          throw new KeePassBBException("ca", "The persistent database was not signed with the proper key.");
      }
			*/
      int len = sdb.db.length;
      if (sdb.dbFieldType != DatabaseObject.FIELD_TYPE_DATA)
        throw new RuntimeException("Record type mismatch: " + sdb.dbFieldType + ":" + DatabaseObject.FIELD_TYPE_DATA + ":" + sdb.dbLength + ":" + len);
      if (len != sdb.dbLength)
        throw new RuntimeException("Actual data array length doesn't match header: " + (sdb.dbLength) + ":" + len);

      uid = sdb.getUID();
      db = sdb.db;
      dbLength = sdb.dbLength;
      dbFieldType = sdb.dbFieldType;
      return;

    } catch (Throwable e) {
      throw new RuntimeException(e.toString());
    }
  }

  public static boolean deletePersistentStore() {
    try {
      PersistentObject store = PersistentStore.getPersistentObject(STORE_UID);
      store.setContents(new DatabaseObject(0), CodeSigningKey.convert("F5EN"), true);
      store.commit();
      return (true);
    } catch (Throwable e) {
      throw new RuntimeException(e.toString());
    }
  }

}
