/*
KeePass for BlackBerry
Copyright 2007 Fairview 5 Engineering, LLC <george.joseph@fairview5.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.fairview5.keepassbb;

import java.io.IOException;

import net.rim.device.api.io.FileNotFoundException;
import net.rim.device.api.system.*;
import net.rim.device.api.ui.*;
import net.rim.device.api.ui.component.*;
import net.rim.device.api.ui.container.MainScreen;

import com.fairview5.keepassbb.file.KFile;
import com.fairview5.keepassbb.kpobjects.*;
import com.fairview5.keepassbb.ui.*;

public final class KeePassBBScreen extends MainScreen implements Runnable, GlobalEventListener {
	KeePassBB uiApp;
	TreeScreen tf;

	public KeePassBBScreen(long style) {
		uiApp = (KeePassBB) UiApplication.getUiApplication();
		onExposed();
	}

	public void run() {
		Font.setDefaultFont(Options.getDefaultFont());
		try {
			tf = new TreeScreen(this);
			tf.setDefaultExpanded(Options.getBooleanOption(Options.OPTION_AUTOEXPANDGROUPS, false));
			add(tf);
			if (Options.getBooleanOption(Options.OPTION_AUTORELOAD, false)) {
				String db = Options.getStringOption(Options.OPTION_DEFAULTDBURL);
				if (db != null && db.length() > 0) {
					if (db.startsWith("http://") || db.startsWith("https://")) {
						this.openHttpDatabase(db);
					} else {
						openDatabase(db);
					}
				} else {
					Dialog.alert("The default database was not specified");
				}
			} else {
				openDatabase();
			}
		} catch (Throwable e1) {
			Dialog.alert(e1.getMessage());
		}
	}

	public void setTitle() {
		if (Options.getBooleanOption(Options.OPTION_HIDETITLE, false))
			super.setTitle((Field) null);
		else {
			if (PwManager.pmCurrent != null) {
				String fn = PwManager.pmCurrent.getFilename();
				if (fn == null) fn = "<unnameddb>";
				fn = fn + (isDirty() ? "*" : "");
				super.setTitle(new LabelField(fn, DrawStyle.ELLIPSIS | DrawStyle.TRUNCATE_BEGINNING));
			} else
				super.setTitle(KeePassBB.name);
		}

	}

	private static String getBackdoor(int bd) {
		StringBuffer sb = new StringBuffer();
		char c = (char) ((bd >>> 24) & 0xff);
		if (c != 0)
			sb.append(c);
		c = (char) ((bd >>> 16) & 0xff);
		if (c != 0)
			sb.append(c);
		c = (char) ((bd >>> 8) & 0xff);
		if (c != 0)
			sb.append(c);
		c = (char) ((bd >>> 0) & 0xff);
		if (c != 0)
			sb.append(c);
		return sb.toString();
	}

	protected boolean openProductionBackdoor(int backdoorCode) {
		String bd = getBackdoor(backdoorCode);
		if (bd.equals("OPTN")) {
			UiApplication.getUiApplication().invokeLater(new Runnable() {
				public void run() {
					miOptions.run();
				}
			});
			return true; // handled
		}
		if (bd.equals("PSTO")) {
			UiApplication.getUiApplication().invokeLater(new Runnable() {
				public void run() {
					tryLegacy();
				}
			});
			return true; // handled
		}
		if (bd.equals("PSTD")) {
			UiApplication.getUiApplication().invokeLater(new Runnable() {
				public void run() {
					delPersistentStore();
				}
			});
			return true; // handled
		}
		return super.openProductionBackdoor(backdoorCode);
	}

	protected boolean onSavePrompt() {
		if (Options.getBooleanOption(Options.OPTION_AUTOSAVECLOSE, true)) {
			return onSave();
		}
		return super.onSavePrompt();
	}

	public void save() {
		if (!PwManager.isValid() || PwManager.isDefaultLegacy() || !isDirty())
			return;

		try {
			final ProgressDialog pd = new ProgressDialog("Something"); 
			final Throwable[] ex = new Throwable[1];
			Thread t = new Thread(new Runnable() {
				public void run() {
					try {
						Utils.logger("starting save");
						PwManager.pmCurrent.save(pd);
						Utils.logger("finishing save");
						pd.close();
						Utils.logger("closed pg");
					} catch (Throwable e) {
						ex[0] = e;
					}
				}
			});
			Utils.logger("starting thread");
			t.start();
			pd.doModal();
			if (ex[0] != null)
				throw ex[0];
			setTitle();
		} catch (Throwable e1) {
			throw new RuntimeException(e1.getMessage());
		}
	}

	public boolean isDirty() {
		return isManagerDirty();
	}

	protected void onExposed() {
		setTitle();
		// setFont(Options.getDefaultFont());
	}

	public boolean isManagerDirty() {
		return (PwManager.isValid() && PwManager.isDefaultDirty());
	}

	public boolean keyChar(char c, int status, int time) {
		if (c == Characters.ESCAPE && Options.getBooleanOption(Options.OPTION_ESCAPE, false)) {
			miHide.run();
			return (true);
		}
		return super.keyChar(c, status, time);
	}

	void addMI(boolean condition, Menu menu, MenuItem mi, int ordinal,
			int priority) {
		if (condition) {
			mi.setOrdinal(ordinal);
			mi.setPriority(priority);
			menu.add(mi);
			return;
		}
	}
	
	
	protected void makeContextMenu(ContextMenu ctx) {
	}

	public void makeMenu(Menu menu, int instance) {

		tf.makeMenu(menu);
		menu.add(MenuItem.separator(500));

		addMI(true, menu, miNewDB, 1000, 0);
		addMI(true, menu, miOpenDB, 1010, 0);
		addMI(true, menu, miOpenURL, 1020, 0);
		addMI(PwManager.isValid() && !PwManager.isDefaultLegacy(), menu, miSave, 1030, 0);
		addMI(PwManager.isValid(), menu, miSaveAs, 1040, 0);
		addMI(PwManager.isValid() && !PwManager.isDefaultLegacy() && !Options.isReadonly(), menu, miMasterKey, 1050, 0);
		addMI(PwManager.isValid(), menu, miClose, 1060, 0);
		menu.add(MenuItem.separator(2000));
		addMI(true, menu, miHelp, 2010, 0);
		addMI(PwManager.isValid() || !Options.getBooleanOption(Options.OPTION_RESTRICTEDMODE, false), menu, miOptions, 2020, 0);
		addMI(true, menu, miAbout, 2030, 0);
		menu.add(MenuItem.separator(3000));
		addMI(isDirty(), menu, miDiscard, 3010, 0);
		addMI(Options.getIntOption(Options.OPTION_SAVETYPE, Options.SAVE_MANUALLY) != Options.SAVE_MANUALLY, menu, miHide, 3020, 0);
		addMI(true, menu, miExit, 3030, 0);
	}

	boolean promptedSave() {
		if (isDirty()) {
			int rc = Dialog.ask(Dialog.D_SAVE);
			if (rc == Dialog.SAVE) {
				save();
				return true;
			} else if (rc == Dialog.DISCARD) {
				return true;
		} else if (rc == Dialog.CANCEL) {
			return false;
		}
	}
		return true;
	}

	MenuItem miClose = new MenuItem("Close", 0, 0) {
		public void run() {
			if (promptedSave()) {
				PwManager.pmCurrent = null;
				tf.deleteAll();
				setTitle();
			}
		}
	};

	MenuItem miHelp = new MenuItem("Help", 0, 0) {
		public void run() {
			Utils.showWebPage("/keepassbb/");
		}
	};

	MenuItem miHide = new MenuItem("Hide", 0, 0) {
		public void run() {
			if (Options.getBooleanOption(Options.OPTION_AUTOSAVEHIDE, true)) {
				save();
			}
			UiApplication.getUiApplication().requestBackground();
		}
	};
	MenuItem miExit = new MenuItem("Exit", 0, 0) {
		public void run() {
			close();
		}
	};

	MenuItem miOpenDB = new MenuItem("Open Database", 0, 0) {
		public void run() {
			openDatabase();
		}
	};
	MenuItem miOpenURL = new MenuItem("Open DB from URL", 0, 0) {
		public void run() {
			openHttpDatabase();
		}
	};

	MenuItem miSaveAs = new MenuItem("Save As", 0, 0) {
		public void run() {
			KFile f = null;
			try {
				PathDialog pd = new PathDialog("Save As", "/", true);
				if (pd.doModal() != Dialog.OK)
					return;
				f = pd.getSelectedFileObject();
				PwManager.pmCurrent.saveAs(f, true);
				PwManager.setDefaultLegacy(false);
				setTitle();
			} catch (Throwable e1) {
				Dialog.alert(e1.getMessage());
			} finally {
				Utils.closeStream(f);
			}
		}
	};

	MenuItem miSave = new MenuItem("Save", 0, 0) {
		public void run() {
			try {
				save();
			} catch (Throwable e1) {
				Dialog.alert(e1.getMessage());
			}
		}
	};

	MenuItem miDiscard = new MenuItem("Discard Changes", 0, 0) {
		public void run() {
			if (PwManager.isValid())
				PwManager.setDefaultDirty(false);
		}
	};
	

	MenuItem miOptions = new MenuItem("Options", 0, 0) {
		public void run() {
			Options sc = new Options();
			uiApp.pushModalScreen(sc);
		}
	};

	MenuItem miAbout = new MenuItem("About", 0, 0) {
		public void run() {
			Status.show(KeePassBB.name + " build " + KeePassBB.version);
			if (PwManager.isValid()) {
				StatusDialog sd = new StatusDialog("Statistics");
				sd.add(new LabelField("Groups: " + PwManager.pmCurrent.getGroupsTable().size()));
				sd.add(new LabelField("Entries: " + PwManager.pmCurrent.getEntriesTable().size()));
				uiApp.pushScreen(sd);
			}
		}
	};

	MenuItem miMasterKey = new MenuItem("Change Master Key", 0, 0) {
		public void run() {
			try {
				if (!PwManager.pmCurrent.setMasterPassword())
					return;
				PwManager.setDefaultDirty(true);

			} catch (Throwable e1) {
				Dialog.alert(e1.getMessage());
				Dialog.alert("Cancelling");
			}
		}
	};

	MenuItem miNewDB = new MenuItem("New Database", 0, 0) {
		public void run() {
			tf.deleteAll();
			KFile f = null;
			try {
				PwManager pwm = new PwManager();
				if (pwm.setMasterPassword()) {
					PwGroup gen = new PwGroup(pwm.rootGroup, "General",
							48, 0);
					gen.add(pwm);
					new PwGroup(gen, "Windows", 38, 1).add(pwm);
					new PwGroup(gen, "Network", 3, 1).add(pwm);
					new PwGroup(gen, "Internet", 1, 1).add(pwm);
					new PwGroup(gen, "eMail", 19, 1).add(pwm);
					new PwGroup(gen, "Homebanking", 37, 1).add(pwm);

					PathDialog pd = new PathDialog("New Database:", "/", true);
					if (pd.doModal() != Dialog.OK)
						return;
					f = pd.getSelectedFileObject();
					PwManager.pmCurrent = pwm;
					pwm.saveAs(f, true);
					f.close();
					PwManager.setDefaultLegacy(false);
					setTitle();
				}
			} catch (Throwable e1) {
				Dialog.alert("Unable to create new database");
			} finally {

				Utils.closeStream(f);
			}
			if (PwManager.isValid())
				tf.loadTree(PwManager.pmCurrent);

		}
	};

	private boolean tryLegacy() {
		byte[] fileContents = null;
		PwManager newp = null;
		try {
			DatabaseObject dbo = new DatabaseObject();
			fileContents = dbo.db;
			newp = new PwManager();
			newp.setLegacy(true);
		} catch (Throwable t) {
			Dialog.alert("No legacy database found");
			return false;
		}
		try {
			Dialog.alert("You are opening a stored database from an earlier version of KeePassBB. You must save it to a file on the device filesystem before you can make any changes to it.  Use the 'Save As' menu item. ");
			if (newp.unserializeWithPrompt(fileContents)) {
				PwManager.pmCurrent = newp;
				PwManager.setDefaultDirty(false);
				setTitle();
				tf.deleteAll();
				tf.loadTree(PwManager.pmCurrent);
				return true;
			}
		} catch (Throwable e) {
			Dialog.alert(e.getMessage());
		}
		return false;

	}

	public boolean openDatabase() {
		KFile f = null;
		try {
			PathDialog pd = new PathDialog("Open Database:", "/", false);
			if (pd.doModal() != Dialog.OK)
				return false;
			f = pd.getSelectedFileObject();
			return openDatabase(f);
		} catch (Exception e) {
			Dialog.alert(e.getMessage());
		} finally {
			Utils.closeStream(f);
		}
		return false;
	}

	public boolean openDatabase(String filename) {
		KFile f = null;
		try {
			f = new KFile(filename);
			return openDatabase(f);
		} catch (Exception e) {
			Dialog.alert(e.getMessage()+": "+filename);
		} finally {
			Utils.closeStream(f);
		}
		return false;
	}

	public boolean openDatabase(KFile f) {
		try {
			if (!f.exists()) throw new FileNotFoundException("File not found: "+f.getFullName());
			PwManager newp = new PwManager();
			if (!newp.load(f))
				return false;
			PwManager.pmCurrent = newp;
			PwManager.setDefaultDirty(false);
			PwManager.setDefaultLegacy(false);
			setTitle();
		} catch (Exception e) {
			Dialog.alert(e.getMessage());
			return false;
		}
		tf.deleteAll();
		tf.loadTree(PwManager.pmCurrent);
		return true;
	}

	public void eventOccurred(long guid, int i, int i1, Object object, Object object1) {
		if (guid == PwSyncManager.GUID_SYNCCOMPLETE) {
			uiApp.invokeLater(new Runnable() {
				public void run() {
					Status.show("Sync complete", 2000);
					try {
						// reloadDatabase();
					} catch (Throwable e1) {
						Dialog.alert(e1.getMessage());
					}
				}
			});
		} else if (guid == PwSyncManager.GUID_SYNCSTART) {
			RuntimeStore rt = RuntimeStore.getRuntimeStore();
			try {
				if (isManagerDirty())
					save();
				rt.put(PwSyncManager.GUID_SYNCPROCEED, "OK");
			} catch (RuntimeException e) {
				rt.put(PwSyncManager.GUID_SYNCPROCEED, e.toString());
			}

		}

	}

	void openHttpDatabase(String url) {
		try {
			boolean ds = Options.getBooleanOption(Options.OPTION_DISABLE_MDS, false);
			byte[] ba = Utils.getHttpFile(url + (ds ? ";deviceside=true":""));

			String fn = "/store/home/user/keepass/httpdb.kdb";

			KFile kf = new KFile(fn);
			if (kf.exists())
				kf.truncate();
			else
				kf.create(true);
			kf.write(ba);
			Utils.closeStream(kf);

			openDatabase(fn);

		} catch (IOException e) {
			Dialog.alert(e.getMessage());
		}

		
	}
	
	void openHttpDatabase() {
		EditDialog ed = new EditDialog("HTTP URL:", new EditField(EditField.FILTER_URL));
		if (ed.doModal() != Dialog.OK)
			return;
		openHttpDatabase(ed.getText());
	}

	void delPersistentStore() {
		try {
			int rc = Dialog.ask(Dialog.D_DELETE, "Are you sure you want to delete the legacy database?");
			if (rc != Dialog.DELETE)
				return;
			PersistentStore.destroyPersistentObject(DatabaseObject.STORE_UID);
			PersistentStore.destroyPersistentObject(Options.STORE_UID_OLD);
			Status.show("Legacy Database Deleted.");
		} catch (Throwable e1) {
			Dialog.alert(e1.getMessage());
		}
	}

}
