/*
KeePass for BlackBerry
Copyright 2007,2008 Fairview 5 Engineering, LLC <george.joseph@fairview5.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.fairview5.keepassbb;

import java.io.*;

import javax.microedition.io.*;
import javax.microedition.io.file.FileConnection;

import net.rim.blackberry.api.browser.Browser;
import net.rim.device.api.io.FileNotFoundException;
import net.rim.device.api.system.*;
import net.rim.device.api.util.NumberUtilities;

import com.fairview5.keepassbb.file.KFile;
import com.fairview5.keepassbb.ui.StatusDialog;

public class Utils {

	public static String printBA(String msg, byte[] ba) {
		return Utils.printBA(msg, ba, 0, ba.length);
	}

	public static String printBA(byte[] ba) {
		return Utils.printBA(null, ba, 0, ba.length);
	}

	public static String printBA(String msg, byte[] ba, int offset, int len) {
		StringBuffer sb = new StringBuffer();
		if (msg != null)
			sb.append(msg);
		for (int i = offset; i < offset + len; i++) {
			NumberUtilities.appendNumber(sb, ba[i], 16, 2);
		}
		return sb.toString();
	}

	public static void closeStream(OutputStream out) {
		if (out == null)
			return;
		try {
			out.flush();
		} catch (Exception e) {
		}
		try {
			out.close();
		} catch (Exception e) {
		}
	}

	public static void closeStream(InputStream in) {
		if (in == null)
			return;
		try {
			in.close();
		} catch (Exception e) {
		}
	}

	public static void closeStream(KFile f) {
		if (f == null)
			return;
		try {
			f.close();
		} catch (Throwable e) {
		}
	}

	public static byte[] getHttpFile(final String url) throws IOException {
		if (url == null)
			throw new IllegalArgumentException("URL is null");
		final Throwable[] ex = new Throwable[1];
		final byte[][] ba = new byte[1][];

		final StatusDialog sd = new StatusDialog("Retrieving " + url);
		Thread t = new Thread(new Runnable() {
			public void run() {
				Connection hc = null;
				DataInputStream dis = null;
				try {
					hc = Connector.open(url);
					int size = 0;

					if (hc instanceof FileConnection) {
						if (!((FileConnection) hc).exists())
							throw new FileNotFoundException("Unable to find file " + url);
						dis = ((FileConnection) hc).openDataInputStream();
						size = (int) ((FileConnection) hc).fileSize();
					} else {
						if (((HttpConnection) hc).getResponseCode() != HttpConnection.HTTP_OK)
							throw new FileNotFoundException("Unable to open file " + url+". Respose code: "+ ((HttpConnection) hc).getResponseCode());

						dis = ((HttpConnection) hc).openDataInputStream();
						size = (int) ((HttpConnection) hc).getLength();
					}
					if (size == 0)
						throw new IOException("File size was 0");

					ba[0] = new byte[size];
					dis.readFully(ba[0]);
				} catch (Throwable e) {
					if (e instanceof FileNotFoundException) {
						ex[0] = e;
					} else {
						ex[0] = new RuntimeException(e.toString());
					}
				} finally {
					try {
						if (dis != null)
							dis.close();
						if (hc != null)
							hc.close();
					} catch (IOException e) {
					}
				}
				return;
			}
		});

		t.start();
		try {
			t.join();
		} catch (InterruptedException e) {
		}
		sd.close();
		if (ex[0] != null) {
			if (ex[0] instanceof FileNotFoundException) throw (IOException) ex[0];
			else throw new RuntimeException(ex[0].getMessage());
			
		}
		return (ba[0]);
	}

	public static long createGUID(String s) {
		long h1 = Math.abs(s.hashCode());
		if (KeePassBB.GUID_BASE == 0)
			return (h1 << 32);
		long h = KeePassBB.GUID_BASE | h1;
		return (h);
	}

	public static void logger(String msg, int level) {
		EventLogger.logEvent(KeePassBB.GUID_LOG, msg.getBytes(), level);
	}

	public static void logger(String msg) {
		logger(msg, EventLogger.ALWAYS_LOG);
	}

	public static void saveToRuntimeStore(String s, long guid) {
		Utils.saveToRuntimeStore(s.getBytes(), guid);
	}

	public static byte[] getFromRuntimeStore(long guid) {
		try {
			RuntimeStore rt = RuntimeStore.getRuntimeStore();
			if (!DeviceInfo.isSimulator()) {
				CodeSigningKey csk = CodeSigningKey.get(CodeModuleManager.getModuleHandle("keepassbb"), "F5EN");
				if (csk == null) {
					throw new RuntimeException("Unable to retrieve signing key");
				}
				ControlledAccess ca = rt.getControlledAccess(guid);
				if (ca == null)
					throw new RuntimeException("No record in temp store");
				boolean valid = ca.checkKeys(csk, csk);
				if (!valid) {
					throw new RuntimeException("The RT key was not signed with the proper key.");
				}
			}
			Object o = rt.get(guid);
			byte[] fk = PersistentContent.decodeByteArray(o);
			return (fk);
		} catch (Throwable e) {
			throw new RuntimeException(e.getMessage());
		}
	}

	public static void saveToRuntimeStore(byte[] fk, long guid) {
		try {
			RuntimeStore rt = RuntimeStore.getRuntimeStore();
			Object fo = fk;
			if (!DeviceInfo.isSimulator()) {
				CodeSigningKey csk = CodeSigningKey.get(CodeModuleManager.getModuleHandle("keepassbb"), "F5EN");
				if (csk == null)
					throw new RuntimeException("Unable to retrieve signing key");
				
				fo = PersistentContent.encode(fk, PersistentContent.isCompressionEnabled(), PersistentContent
						.isEncryptionEnabled());
				fo = new ControlledAccess(fo, csk);
			}
			rt.replace(guid, fo);

		} catch (Exception e) {
			throw new RuntimeException(e.getMessage());
		}

	}

	public static boolean isHexDigit(char c) {
		return c >= '0' && c <= '9' || c >= 'a' && c <= 'f' || c >= 'A' && c <= 'F';
	}

	public static boolean isHexString(String s) {
		int l = s.length();
		for (int i = 0; i < l; i++) {
			if (!isHexDigit(s.charAt(i)))
				return false;
		}
		return true;
	}

	public static String getScheme(String url) {
		int ix = url.indexOf(':');
		if (ix <= 0)
			return null;
		return (url.substring(0, ix));
	}

	public static String getHost(String url) {
		int ix1 = url.indexOf("://");
		if (ix1 <= 0)
			return null;
		ix1 += 3;
		int ix2 = url.indexOf('/', ix1);
		if (ix2 <= 0)
			return url.substring(ix1);
		return (url.substring(ix1, ix2));
	}

	public static String getPath(String url) {
		int ix1 = url.indexOf("://");
		if (ix1 <= 0)
			return null;
		ix1 += 3;
		int ix2 = url.indexOf('/', ix1);
		if (ix2 <= 0)
			return null;
		int ix3 = url.indexOf('#');
		if (ix3 > 0)
			return (url.substring(ix2, ix3));
		int ix4 = url.indexOf('?');
		if (ix4 > 0)
			return (url.substring(ix2, ix4));
		return (url.substring(ix2));
	}

	public static String getRef(String url) {
		int ix3 = url.indexOf('#');
		if (ix3 <= 0)
			return null;
		ix3++;
		int ix4 = url.indexOf('?');
		if (ix4 > 0)
			return (url.substring(ix3, ix4));
		return (url.substring(ix3));
	}

	public static String getQueryString(String url) {
		int ix4 = url.indexOf('?');
		if (ix4 <= 0)
			return null;
		return (url.substring(ix4 + 1));
	}
	
	public static void showWebPage(String path) {
		String host = "f5bbutil.sourceforge.net";
		if (DeviceInfo.isSimulator()) {
			host = "localbb:8090";
		}
		Browser.getDefaultSession().displayPage("http://"+host+path);
		
	}
	
	public static String getOSSoftwareVersionString() {
		int mh = CodeModuleManager.getModuleHandle("net_rim_bb_phone_api");
		return CodeModuleManager.getModuleVersion(mh);
	}
	public static double getOSSoftwareVersion() {
		String s = getOSSoftwareVersionString();
		int ix1 = s.indexOf('.');
		if (ix1 <= 0) return(0);
		int ix2 = s.indexOf('.',ix1+1);
		return Double.parseDouble(s.substring(0,ix2));
	}
}
