/*
KeePass for BlackBerry
Copyright 2007,2008 Fairview 5 Engineering, LLC <george.joseph@fairview5.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package com.fairview5.keepassbb.kpobjects;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.util.Calendar;
import java.util.Date;
import java.util.TimeZone;

import net.rim.device.api.crypto.SHA256Digest;

public class KdbInputStream extends InputStream {
  SHA256Digest md = new SHA256Digest();
  InputStream is;
  String timezone;


  public KdbInputStream(byte[] bytes) {
  	int l = bytes.length;
    is = new ByteArrayInputStream(bytes, 0, l);
  }
  public KdbInputStream(byte[] bytes, int start, int len) {
	    is = new ByteArrayInputStream(bytes, start, len);
	  }

  public KdbInputStream(byte[] bytes, String tz) {
    this(bytes);
    timezone = tz;
  }

  public KdbInputStream(InputStream is) {
    this.is = is;
  }

  public KdbInputStream(InputStream is, String tz) {
    this.is = is;
    timezone = tz;
  }

  public void close() throws IOException {
    is.close();
  }

  public int readInt() throws IOException {
    return (read() & 0xFF) + ((read() & 0xFF) << 8) + ((read() & 0xFF) << 16)
     + ((read() & 0xFF) << 24);
  }

  public int readShort() throws IOException {
    return (read() & 0xFF) + ((read() & 0xFF) << 8);
  }

  public Date readTime() throws IOException {
    int dw1 = read();
    int dw2 = read();
    int dw3 = read();
    int dw4 = read();
    int dw5 = read();

    // Unpack 5 byte structure to date and time
    Calendar time = Calendar.getInstance();
    if (timezone != null) time.setTimeZone(TimeZone.getTimeZone(timezone));

    time.set(Calendar.YEAR, (dw1 << 6) | (dw2 >> 2));
    time.set(Calendar.MONTH, (((dw2 & 0x00000003) << 2) | (dw3 >> 6)) - 1);
    time.set(Calendar.DATE, ((dw3 >> 1) & 0x0000001F) );
    time.set(Calendar.HOUR_OF_DAY, ((dw3 & 0x00000001) << 4) | (dw4 >> 4));
    time.set(Calendar.MINUTE, ((dw4 & 0x0000000F) << 2) | (dw5 >> 6));
    time.set(Calendar.SECOND, dw5 & 0x0000003F);
    return time.getTime();

    //return null;
  }

  public String readString(int len) throws IOException {
    byte[] temp = new byte[len - 1];
    read(temp, 0, temp.length);
    read();
    String s = null;
    try {
      s = new String(temp, "UTF-8");
    } catch (UnsupportedEncodingException e) {

    }
    return (s);
  }

  public int read(byte[] temp, int i, int length) throws IOException {
    int br = is.read(temp, i, length);
    if (br > 0) md.update(temp, i, length);
    return (br);
  }


  public int read(byte[] temp) throws IOException {
    int br = is.read(temp);
    if (br > 0) md.update(temp);
    return (br);
  }

  public int read() throws IOException {
    int i = is.read();
    if (i > -1) md.update(i&0xff);
    return (i);
  }

  public int getDigest(byte[] b, int offset, boolean reset) {
    return(md.getDigest(b, offset, reset));
  }

  public byte[] getDigest() {
    return(md.getDigest());
  }

  public byte[] getDigest(boolean reset) {
    return (md.getDigest(reset));
  }


}

