/*
KeePass for BlackBerry
Copyright 2007 Fairview 5 Engineering, LLC <george.joseph@fairview5.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package com.fairview5.keepassbb.kpobjects;

import java.io.UnsupportedEncodingException;
import java.util.Calendar;
import java.util.Date;
import java.util.TimeZone;

import net.rim.device.api.io.NoCopyByteArrayOutputStream;

public class KdbOutputStream extends NoCopyByteArrayOutputStream {

  String timezone;

  public KdbOutputStream(String tz) {
    super();
    timezone = tz;
  }
  public KdbOutputStream(String tz, int initialSize) {
	    super(initialSize);
	    timezone = tz;
	  }
  public KdbOutputStream(int initialSize) {
	    super(initialSize);
	  }

  public KdbOutputStream() {
    super();
  }

  public void writeIntField(int fieldId, int val) {
    writeShort(fieldId);
    writeInt(4);
    writeInt(val);
  }

  public void writeShortField(int fieldId, int val) {
    writeShort(fieldId);
    writeInt(2);
    writeShort(val);
  }

  public void writeTimeField(int fieldId, Date val) {
    writeShort(fieldId);
    writeInt(5);
    writeTime(val);
  }

  public void writeStringField(int fieldId, String val) {
    byte[] ba = null;
    try {
      ba = val.getBytes("UTF-8");
      writeShort(fieldId);
      writeInt(ba.length+1);
      write(ba, 0, ba.length);
      write(0);
    } catch (UnsupportedEncodingException e) {
      e.printStackTrace();
    }
  }

  public void writeByteArrayField(int fieldId, byte[] val) {
    writeByteArrayField(fieldId, val, 0, val.length);
  }

  public void writeByteArrayField(int fieldId, byte[] val, int offset, int len) {
    writeShort(fieldId);
    writeInt(len);
    write(val, offset, len);
  }

  public void writeEOF() {
    writeShort(0xFFFF);
    writeInt(0);
  }

  public void writeInt(int val) {
    write((byte) (val & 0xFF));
    write((byte) ((val >>> 8) & 0xFF));
    write((byte) ((val >>> 16) & 0xFF));
    write((byte) ((val >>> 24) & 0xFF));
  }

  public void writeShort(int val) {
    write((byte) (val & 0xFF));
    write((byte) ((val >>> 8) & 0xFF));
  }

  public void writeTime(Date d) {
    Calendar c = Calendar.getInstance();
    if (timezone != null) c.setTimeZone(TimeZone.getTimeZone(timezone));
    c.setTime(d);
    write((byte)((c.get(Calendar.YEAR) >> 6) & 0x0000003F));
    write((byte) (((c.get(Calendar.YEAR) & 0x0000003F) << 2) | (((c.get(Calendar.MONTH) + 1) >> 2) & 0x00000003)));
    write((byte) ((((c.get(Calendar.MONTH) + 1) & 0x00000003) << 6) | ((c.get(Calendar.DATE ) & 0x0000001F) << 1) | ((c.get(Calendar.HOUR_OF_DAY) >> 4) & 0x00000001)));
    write((byte) (((c.get(Calendar.HOUR_OF_DAY) & 0x0000000F) << 4) | ((c.get(Calendar.MINUTE) >> 2) & 0x0000000F)));
    write((byte) (((c.get(Calendar.MINUTE) & 0x00000003) << 6) | (c.get(Calendar.SECOND) & 0x0000003F)));
  }


}
