/*
KeePass for BlackBerry
Copyright 2007,2008 Fairview 5 Engineering, LLC <george.joseph@fairview5.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.fairview5.keepassbb.kpobjects;

import net.rim.device.api.crypto.RandomSource;
import net.rim.device.api.crypto.SHA256Digest;
import net.rim.device.api.util.Persistable;

import com.fairview5.keepassbb.Utils;

public class KeyFile implements Persistable {
	//static PersistentObject store;

	private byte[] key = new byte[32];

	public KeyFile() {
		SHA256Digest d = new SHA256Digest();
		d.update(RandomSource.getBytes(1024));
		d.getDigest(getKey(), 0);
	}

	public KeyFile(byte[] k) {
		importKeyText(k);
	}

	public String exportKeyText() {
		return Utils.printBA(getKey());
	}

	public void importKeyText(byte[] ba) {
		if (ba.length == 32) {
			System.arraycopy(ba, 0, key, 0, 32);
			return;
		}
		String fcs = new String(ba);
		if (ba.length == 64 && Utils.isHexString(fcs)) {
			key = new byte[32];
			for (int i = 0, j=0; i < 32; i++, j+=2) {
				key[i] = (byte) (Integer.parseInt(fcs.substring(j, j + 2), 16) & 0xff);
			}
			return;
		}
		SHA256Digest md = new SHA256Digest();
		md.update(ba);
		key = md.getDigest();
	}

	public String toString() {
		return getKey() == null ? "<null>" : Utils.printBA(getKey());
	}

	public void setKey(byte[] key) {
		this.key = key;
	}

	public byte[] getKey() {
		return key;
	}

}
