/*
KeePass for BlackBerry

Copyright 2007 Fairview 5 Engineering, LLC <george.joseph@fairview5.com>

KeePass for J2ME
Copyright 2007 Naomaru Itoi <nao@phoneid.org>

Java clone of KeePass - A KeePass file viewer for Java
Copyright 2006 Bill Zwicky <billzwicky@users.sourceforge.net>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package com.fairview5.keepassbb.kpobjects;

public final class PwDbHeader {

	public static final int BUF_SIZE = 124;
	public static final int ALGO_TWOFISH = 1;
	public static final int ALGO_AES = 0;
	public static final int PWM_FLAG_TWOFISH = 8;
	public static final int PWM_FLAG_ARCFOUR = 4;
	public static final int PWM_FLAG_RIJNDAEL = 2;
	public static final int PWM_FLAG_SHA2 = 1;
	public static final int PWM_DBVER_DW = 0x00030002;
	
	public static final int PWM_DBSIG_1 = 0x9AA2D903;
	public static final int PWM_DBSIG_2 = 0xB54BFB65;


	public int signature1; // = PWM_DBSIG_1
	public int signature2; // = PWM_DBSIG_2
	public int flags;
	public int version;
	public byte masterSeed[] = new byte[16];
	public byte encryptionIV[] = new byte[16];
	public int numGroups;
	public int numEntries;
	public byte contentsHash[] = new byte[32];
	public byte masterSeed2[] = new byte[32];
	public int numKeyEncRounds;

	public PwDbHeader() {

	}

	public void unserialize(KdbInputStream bais) {
		try {
			signature1 = bais.readInt();
			signature2 = bais.readInt();
			flags = bais.readInt();
			version = bais.readInt();
			bais.read(masterSeed, 0, masterSeed.length);
			bais.read(encryptionIV, 0, encryptionIV.length);
			numGroups = bais.readInt();
			numEntries = bais.readInt();
			bais.read(contentsHash, 0, contentsHash.length);
			bais.read(masterSeed2, 0, masterSeed2.length);
			numKeyEncRounds = bais.readInt();
		} catch (Exception e) {
			throw new RuntimeException("Unable to deserialize db header: "
					+ e.getMessage());
		}
	}

	public void serialize(KdbOutputStream baos) {
		try {
			baos.writeInt(signature1);
			baos.writeInt(signature2);
			baos.writeInt(flags);
			baos.writeInt(version);
			baos.write(masterSeed, 0, masterSeed.length);
			baos.write(encryptionIV, 0, encryptionIV.length);
			baos.writeInt(numGroups);
			baos.writeInt(numEntries);
			baos.write(contentsHash, 0, contentsHash.length);
			baos.write(masterSeed2, 0, masterSeed2.length);
			baos.writeInt(numKeyEncRounds);
		} catch (Exception e) {
			throw new RuntimeException("Unable serialize db header: "
					+ e.getMessage());
		}
	}

}
