/*
KeePass for BlackBerry
Copyright 2007 Fairview 5 Engineering, LLC <george.joseph@fairview5.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.fairview5.keepassbb.kpobjects;

import java.io.IOException;
import java.util.Date;

import net.rim.device.api.i18n.SimpleDateFormat;
import net.rim.device.api.ui.*;
import net.rim.device.api.ui.component.*;
import net.rim.device.api.ui.container.HorizontalFieldManager;
import net.rim.device.api.ui.container.MainScreen;

import com.fairview5.keepassbb.Options;
import com.fairview5.keepassbb.ui.*;

public final class PwEntryScreen extends MainScreen {
	static SimpleDateFormat sdf = new SimpleDateFormat("EEE MMM d yyyy  h:mm aa");

	PwEntryScreen me;
	final PwEntry pwe;

	EditField efTitle;
	ActiveAutoTextEditField efURL;
	EditField efUsername;
	TextField efPassword;
	ActiveAutoTextEditField efNotes;

	DateField dfCreated;
	DateField dfExpires;
	DateField dfLastAccess;
	DateField dfLastMod;

	IconChooserWidget icwIcon;

	int rc = Dialog.CANCEL;

	public PwEntryScreen(final PwEntry e) {
		me = this;
		this.pwe = e;

		if (Options.getBooleanOption(Options.OPTION_HIDETITLE, false))
			setTitle((Field) null);
		else
			setTitle(e.title);

		long attr = 0;

		if (Options.getBooleanOption(Options.OPTION_READONLY, false) || PwManager.isDefaultLegacy())
			attr = Field.READONLY;
		else
			attr = TextField.EDITABLE;
		
		efTitle = new EditField("Title:", e.title, TextField.DEFAULT_MAXCHARS,
				attr | EditField.NO_NEWLINE);
		efURL = new ActiveAutoTextEditField("URL:", e.url,
				TextField.DEFAULT_MAXCHARS, attr | EditField.NO_NEWLINE);
		efUsername = new EditField("User:", e.username,
				TextField.DEFAULT_MAXCHARS, attr | EditField.NO_NEWLINE);
		efPassword = new EditField("Password:", e.password,
				TextField.DEFAULT_MAXCHARS, attr | EditField.NO_NEWLINE | TextField.NO_COMPLEX_INPUT | TextField.NO_LEARNING);
		efNotes = new ActiveAutoTextEditField("Notes:", e.additional,
				TextField.DEFAULT_MAXCHARS, attr);

		add(efTitle);

		if (!Options.getBooleanOption(Options.OPTION_HIDEENTRYURL, false))
			add(efURL);

		add(efUsername);
		add(efPassword);
		add(efNotes);
		add(new SeparatorField());
		icwIcon = new IconChooserWidget(TreeScreen.mIcon, e.imageId);
		add(icwIcon);
		add(new SeparatorField());
		attr = 0;
		HorizontalFieldManager h1 = new HorizontalFieldManager(attr);
		HorizontalFieldManager h2 = new HorizontalFieldManager(attr);
		HorizontalFieldManager h3 = new HorizontalFieldManager(attr);
		HorizontalFieldManager h4 = new HorizontalFieldManager(attr);

		attr = DrawStyle.RIGHT;
		h1.add(new FixedWidthLabelField("Created:", "Last Mod:", attr));
		h2.add(new FixedWidthLabelField("Expires:", "Last Mod:", attr));
		h3.add(new FixedWidthLabelField("Last Acc:", "Last Mod:", attr));
		h4.add(new FixedWidthLabelField("Last Mod:", "Last Mod:", attr));

		attr = Manager.USE_ALL_WIDTH | DrawStyle.RIGHT | DateField.DATE_TIME;
		dfCreated = new DateField(null, e.tCreation.getTime(), sdf, attr | Field.READONLY);
		dfExpires = new DateField(null, e.tExpire.getTime(), sdf, attr);
		dfLastAccess = new DateField(null, e.tLastAccess.getTime(), sdf, attr | Field.READONLY);
		dfLastMod = new DateField(null, e.tLastMod.getTime(), sdf, attr | Field.READONLY);
		h1.add(dfCreated);
		h2.add(dfExpires);
		h3.add(dfLastAccess);
		h4.add(dfLastMod);

		add(h1);
		add(h2);
		add(h3);
		add(h4);

		e.tLastAccess = new Date();
		//if (Options.getBooleanOption(Options.OPTION_SAVELASTACCESS, false)) setDirty(true);		
	}

	public void makeMenu(Menu km, int instance) {
		if (isDirty()) {
			km.add(new MenuItem("Discard Changes", 100, 0) {
				public void run() {
					me.setDirty(false);
					close();
				}
			});
			km.add(new MenuItem("Save", 120, 0) {
				public void run() {
					me.setDirty(false);
					onSave();
				}
			});
		}

		km.add(new MenuItem("Close", 130, 0) {
			public void run() {
				close();
			}
		});
	}

	public boolean isDataValid() {
		if (efTitle.getText().length() <= 0) {
			Dialog.alert("Title can't be empty.");
			return (false);
		}
		return (true);
	}

	protected boolean onSavePrompt() {

		if (Options.getBooleanOption(Options.OPTION_AUTOACCEPT, false)) {
			return onSave();
		}
		if (!super.isDirty() && Options.getBooleanOption(Options.OPTION_SAVELASTACCESS, false))
			return onSave();

		return super.onSavePrompt();
	}

	public void save() {
		pwe.title = efTitle.getText();
		pwe.url = efURL.getText();
		pwe.username = efUsername.getText();
		pwe.password = efPassword.getText();
		pwe.additional = efNotes.getText();
		pwe.tLastMod = new Date();
		pwe.tExpire = new Date(dfExpires.getDate());
		pwe.imageId = icwIcon.getSelectedIcon();
		rc = Dialog.OK;
		PwManager.setDefaultDirty(true);
		if (Options.getBooleanOption(Options.OPTION_AUTOSAVECHANGES, true)) {
			try {
				PwManager.pmCurrent.save(true);
			} catch (IOException e) {
				Dialog.alert(e.getMessage());
			}
		}

	}

	public void show() {
		UiApplication.getUiApplication().pushScreen(this);
	}

	public int doModal() {
		UiApplication.getUiApplication().pushModalScreen(this);
		return (rc);
	}

}
