/*
KeePass for BlackBerry
Copyright 2007 Fairview 5 Engineering, LLC <george.joseph@fairview5.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.fairview5.keepassbb.kpobjects;

import java.io.IOException;
import java.util.Date;

import net.rim.device.api.i18n.SimpleDateFormat;
import net.rim.device.api.ui.*;
import net.rim.device.api.ui.component.*;
import net.rim.device.api.ui.container.HorizontalFieldManager;
import net.rim.device.api.ui.container.MainScreen;

import com.fairview5.keepassbb.Options;
import com.fairview5.keepassbb.ui.*;

public class PwGroupScreen extends MainScreen {
	static SimpleDateFormat sdf = new SimpleDateFormat("EEE MMM d yyyy  h:mm aa");

	PwGroup pwg;

	EditField efName;
	DateField dfCreated;
	DateField dfExpires;
	DateField dfLastAccess;
	DateField dfLastMod;
	IconChooserWidget icwIcon;
	int rc = Dialog.CANCEL;

	public PwGroupScreen(final PwGroup g) {
		this.pwg = g;

		if (Options.getBooleanOption(Options.OPTION_HIDETITLE, false))
			setTitle((Field) null);
		else
			setTitle(pwg.name);

		long attr = 0;

		if (Options.getBooleanOption(Options.OPTION_READONLY, false) || PwManager.isDefaultLegacy())
			attr = READONLY;
		efName = new EditField("Name:", g.name, TextField.DEFAULT_MAXCHARS,
				attr | EditField.NO_NEWLINE);

		add(efName);
		add(new SeparatorField());
		icwIcon = new IconChooserWidget(TreeScreen.mIcon, g.imageId);
		add(icwIcon);
		add(new SeparatorField());
		attr = 0;
		HorizontalFieldManager h1 = new HorizontalFieldManager(attr);
		HorizontalFieldManager h2 = new HorizontalFieldManager(attr);
		HorizontalFieldManager h3 = new HorizontalFieldManager(attr);
		HorizontalFieldManager h4 = new HorizontalFieldManager(attr);

		attr = DrawStyle.RIGHT;
		h1.add(new FixedWidthLabelField("Created:", "Last Mod:", attr));
		h2.add(new FixedWidthLabelField("Expires:", "Last Mod:", attr));
		h3.add(new FixedWidthLabelField("Last Acc:", "Last Mod:", attr));
		h4.add(new FixedWidthLabelField("Last Mod:", "Last Mod:", attr));

		attr = Manager.USE_ALL_WIDTH | DrawStyle.RIGHT | DateField.DATE_TIME;
		dfCreated = new DateField(null, g.tCreation.getTime(), sdf, attr | Field.READONLY);
		dfExpires = new DateField(null, g.tExpire.getTime(), sdf, attr);
		dfLastAccess = new DateField(null, g.tLastAccess.getTime(), sdf, attr | Field.READONLY);
		dfLastMod = new DateField(null, g.tLastMod.getTime(), sdf, attr | Field.READONLY);
		h1.add(dfCreated);
		h2.add(dfExpires);
		h3.add(dfLastAccess);
		h4.add(dfLastMod);

		add(h1);
		add(h2);
		add(h3);
		add(h4);

		g.tLastAccess = new Date();
	}

	public void makeMenu(Menu km, int instance) {
		if (isDirty()) {
			km.add(new MenuItem("Discard Changes", 100, 0) {
				public void run() {
					setDirty(false);
					close();
				}
			});
		}
		if (isDirty()) {
			km.add(new MenuItem("Save", 110, 0) {
				public void run() {
					setDirty(false);
					onSave();
				}
			});
		}
		km.add(new MenuItem("Close", 120, 0) {
			public void run() {
				close();
			}
		});

	}

	public boolean isDataValid() {
		if (efName.getText().length() <= 0) {
			Dialog.alert("Name can't be empty");
			return (false);
		}
		return (true);
	}

	protected boolean onSavePrompt() {

		if (Options.getBooleanOption(Options.OPTION_AUTOACCEPT, false)) {
			return onSave();
		}

		if (!super.isDirty() && Options.getBooleanOption(Options.OPTION_SAVELASTACCESS, false))
			return onSave();

		return super.onSavePrompt();
	}

	public boolean isDirty() {
		boolean d = super.isDirty();
		if (d || Options.getBooleanOption(Options.OPTION_SAVELASTACCESS, false)) {
			return true;
		}
		return false;
	}

	public void save() {
		pwg.name = efName.getText();
		pwg.tLastMod = new Date();
		pwg.tExpire = new Date(dfExpires.getDate());
		pwg.imageId = icwIcon.getSelectedIcon();
		rc = Dialog.OK;
		PwManager.setDefaultDirty(true);
		if (Options.getBooleanOption(Options.OPTION_AUTOSAVECHANGES, true)) {
			try {
				PwManager.pmCurrent.save(true);
			} catch (IOException e) {
				Dialog.alert(e.getMessage());
			}
		}
	}

	public void show() {
		UiApplication.getUiApplication().pushScreen(this);
	}

	public int doModal() {
		UiApplication.getUiApplication().pushModalScreen(this);
		return (rc);
	}

}
