/*
KeePass for BlackBerry
Copyright 2007,2008 Fairview 5 Engineering, LLC <george.joseph@fairview5.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.fairview5.keepassbb.kpobjects;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Date;

import net.rim.device.api.i18n.Locale;
import net.rim.device.api.io.FileNotFoundException;
import net.rim.device.api.synchronization.*;
import net.rim.device.api.system.*;
import net.rim.device.api.util.*;

import com.fairview5.keepassbb.Options;
import com.fairview5.keepassbb.Utils;
import com.fairview5.keepassbb.file.ByteArray;
import com.fairview5.keepassbb.file.KFile;

public class PwSyncManager implements SyncCollection, SyncConverter {
	
	private static final int RC_DEVICE_FILE_SERIALIZE_EXCEPTION = 12;
	private static final int RC_NO_CHANGES_MADE = 1;
	private static final int RC_CHANGES_MADE = 2;
	private static final int RC_RECONCILE_EXCEPTION = 3;
	private static final int RC_DESKTOP_DECRYPT_FAILURE = 4;
	private static final int RC_DEVICE_DECRYPT_FAILURE = 5;
	private static final int RC_CONVERT_OUT_ERROR = 6;
	private static final int RC_CONVERT_IN_ERROR = 7;
	private static final int RC_RESTORE = 8;
	private static final int RC_BACKUP = 9;
	private static final int RC_LOADED_TO_DEVICE = 10;
	private static final int RC_DEVICE_FILE_OPEN_EXCEPTION = 11;

	public static final int FIELD_RECORD_TYPE = 0;

	public static final long GUID_SYNCSTART = Utils.createGUID("GUID_SYNCSTART");
	public static final long GUID_SYNCPROCEED = Utils
			.createGUID("GUID_SYNCPROCEED");
	public static final long GUID_SYNCCOMPLETE = Utils
			.createGUID("GUID_SYNCCOMPLETE");
	static final long GUID_PASSWORD = Utils.createGUID("GUID_PASSWORD");
	public static final long GUID_MASTERKEY = Utils.createGUID("GUID_MASTERKEY");
	public static final long GUID_FINALKEY = Utils.createGUID("GUID_FINALKEY");

	private PwSyncObject dboDX;
	private byte[] masterKey;
	private static StringBuffer sbProgress = null;
	private boolean inTransaction = false;
	String password;
	PwSyncMessage psMessage = new PwSyncMessage();
	PwSyncCredentials psCredentials;

	private static void progress(String msg) {
		if (sbProgress != null)
			sbProgress.append(msg + '\n');
		else
			Utils.logger(msg);
	}

	private void progress(int rc, final String msg) {
		progress(msg);
		psMessage.message = msg;
		psMessage.rc = rc;
		/*
		 * Application.getApplication().invokeLater(new Runnable() { public void
		 * run() { EditDialog ed = new EditDialog("KeePass for BlackBerry");
		 * ed.addField(new LabelField(msg)); Ui.getUiEngine().pushGlobalScreen(ed,
		 * 0, UiEngine.GLOBAL_MODAL); } })
		 */;
	}

	public boolean convert(SyncObject syncObject, DataBuffer dataBuffer,
			int version) {
		progress("Convert "+syncObject.getClass().getName()+" -> DataBuffer");

		try {
			if (syncObject instanceof PwSyncMessage) {
				ConverterUtilities.writeInt(dataBuffer, FIELD_RECORD_TYPE, PwSyncMessage.UID);
				return ((PwSyncMessage) syncObject).convert(dataBuffer);
			}
			if (syncObject instanceof PwSyncObject) {
				PwSyncObject dbo = (PwSyncObject) syncObject;
				ConverterUtilities.writeInt(dataBuffer, FIELD_RECORD_TYPE, PwSyncObject.UID);
				return dbo.convert(dataBuffer);
			}
		} catch (Exception e) {
			progress(RC_CONVERT_OUT_ERROR, "KeePassBB convert OUT " + e.toString());
		}
		return false;
	}

	public SyncObject convert(DataBuffer dataBuffer, int i, int xuid) {
		if (i != this.getSyncVersion()) {
			throw new RuntimeException("INCOMPATABLE VERSION");
		}
		try {
			int uid = ConverterUtilities.readInt(dataBuffer);
			switch (uid) {
				case PwSyncCredentials.UID:
					progress("Convert PwSyncCredentials from buffer");
					return new PwSyncCredentials(dataBuffer);
				case PwSyncObject.UID:
					progress("Convert PwSyncObject from buffer");
					return new PwSyncObject(dataBuffer);
			}
		} catch (Throwable e) {
			progress(RC_CONVERT_IN_ERROR, "KeePassBB convert IN " + e.toString());
		}
		return null;
	}

	public boolean addSyncObject(SyncObject syncObject) {
		progress("AddSyncObject "+syncObject.getClass().getName());
		if (syncObject instanceof PwSyncObject) {
			dboDX = (PwSyncObject) syncObject;
			return true;
		}
		if (syncObject instanceof PwSyncMessage) {
			psMessage = (PwSyncMessage) syncObject;
			return true;
		}
		if (syncObject instanceof PwSyncCredentials) {
			psCredentials = (PwSyncCredentials) syncObject;
			return true;
		}
		return false;
	}

	public boolean updateSyncObject(SyncObject syncObject, SyncObject syncObject1) {
		progress("UpdateSyncObject "+syncObject.getClass().getName()+":"+syncObject.getUID()+":"+syncObject1.getUID());
		return false;
	}

	public boolean removeSyncObject(SyncObject syncObject) {
		progress("RemoveSyncObject "+syncObject.getClass().getName()+":"+syncObject.getUID());
		return false;
	}

	public boolean removeAllSyncObjects() {
		progress(RC_RESTORE, "Restore started.");
		return false;
	}

	public SyncObject[] getSyncObjects() {
		SyncObject psa[];
		if (!inTransaction) {
			progress(RC_BACKUP,"Backup started.");
			try {
				String filename = Options.getStringOption(Options.OPTION_DEFAULTDBURL);
				PwSyncObject ps = new PwSyncObject(filename);
				psa = new PwSyncObject[1];
				psa[0] = ps;
			} catch (FileNotFoundException e) {
				psa = new PwSyncObject[0];
			}
			return psa;
		}
		if (dboDX == null) {
			progress("getSyncObjects PwSyncMessage");
			psa = new SyncObject[] { psMessage };
		} else {
			progress("getSyncObjects PwSyncObject PwSyncMessage");
			psa = new SyncObject[] {  dboDX, psMessage, };
		}
		return psa;
	}

	public SyncObject getSyncObject(int i) {
		if (!inTransaction) {
			try {
				progress("getSyncObject NIT PwSyncObject");
				String filename = Options.getStringOption(Options.OPTION_DEFAULTDBURL);
				return (new PwSyncObject(filename));
			} catch (FileNotFoundException e) {
				throw new RuntimeException(e.getMessage());
			}
		}
		switch (i) {
		case (0):
			progress("getSyncObject PwSyncObject");
			return dboDX;
		case (1):
			progress("getSyncObject PwSyncMessage");
			return psMessage;
		}
		return (null);
	}

	public boolean isSyncObjectDirty(SyncObject syncObject) {
		progress("isSyncObjectDirty "+syncObject.getClass().getName()+":"+syncObject.getUID());
		if (syncObject instanceof PwSyncMessage) {
			return (true);
		}
		if (!inTransaction) {
			progress("isSyncObjectDirty Not In Transaction");
			return (true);
		}
		PwSyncObject dboDesktop = (PwSyncObject) syncObject;
		PwManager pwmDesktop;
		PwManager pwmDevice;

		PwSyncObject dboLocal;
		try {
			String filename = Options.getStringOption(Options.OPTION_DEFAULTDBURL);
			
			if (filename == null || filename.length() == 0) {
				dboDX.filename = PwSyncObject.generateDefaultFilename();
				Options.setOption(Options.OPTION_AUTORELOAD, true);
				Options.setOption(Options.OPTION_DEFAULTDBURL, dboDX.filename);
				String msg = "No database was found on the device.  The desktop version will be loaded to "+dboDX.filename +
				(psCredentials.keyfile != null ? " For security reasons, your keyfile must be transferred to the device manually. " : "");
				progress(RC_LOADED_TO_DEVICE, msg);
				return false;
			}
			dboDX.filename = filename;
			KFile kf = new KFile(filename);
			if (!kf.exists()) {
				String msg = "No database was found on the device.  The desktop version will be loaded to "+dboDX.filename +
				(psCredentials.keyfile != null ? " For security reasons, your keyfile must be transferred to the device manually. " : "");
				progress(RC_LOADED_TO_DEVICE, msg);
				kf.close();
				return false;
			}
			kf.close();
			dboLocal = new PwSyncObject(filename);
		} catch (Exception e) {
			e.printStackTrace();
			progress(RC_DEVICE_FILE_OPEN_EXCEPTION, "Exception opening the device database: "+e.toString());
			return false;
		}
		
		KeyFile kf = null;
		if (psCredentials.keyfile != null) kf = new KeyFile(psCredentials.keyfile);
		masterKey = PwManager.createMasterKey(psCredentials.password, kf);

		try {
			pwmDevice = new PwManager(dboLocal.db, masterKey);
		} catch (Exception e) {
			progress(RC_DEVICE_DECRYPT_FAILURE, "Unable to decrypt device database using the supplied credentials");
			return false;
		}

		try {
			pwmDesktop = new PwManager(dboDX.db, masterKey);
		} catch (Exception e) {
			progress(RC_DESKTOP_DECRYPT_FAILURE, "Unable to decrypt desktop database using the supplied credentials");
			return false;
		}

		boolean dirty = false;

		try {
			dirty = reconcileGroups(pwmDesktop, pwmDevice)
					| reconcileEntries(pwmDesktop, pwmDevice);
		} catch (Exception e) {
			progress(RC_RECONCILE_EXCEPTION, "There was an error while reconciling databases: " + e.toString());
			return false;
		}

		if (dirty) {
			progress(RC_CHANGES_MADE, "Changes made");
			try {
				ByteArray ba = pwmDesktop.serialize(null);
				dboDesktop.db = ba.byteArray;
				dboDesktop.dbLength = ba.length;
			} catch (Exception e) {
				progress(RC_DEVICE_FILE_SERIALIZE_EXCEPTION, "Exception while serializing database: " + e.toString());
				return false;
			}
		} else {
			progress(RC_NO_CHANGES_MADE, "No changed detected.");
		}
		progress("Sync successful.");
		return dirty;
	}

	private boolean reconcileEntries(PwManager pwmDesktop, PwManager pwmDevice) {

		boolean dirty = false;
		progress("Reconciling entries\n");

		PwEntry[] desktopEntries = pwmDesktop.getEntriesTable()
				.sortedEntryObjects();
		if (desktopEntries != null) {
			for (int i = 0; i < desktopEntries.length; i++) {
				if (!pwmDevice.getEntriesTable()
						.containsKey(desktopEntries[i].uuid)) {
					dirty = true;
					progress("Desktop->Device adding entry "
							+ desktopEntries[i].title + "\n");
					desktopEntries[i].add(pwmDevice);
				}
			}
		}

		PwEntry[] deviceEntries = pwmDevice.getEntriesTable()
				.sortedEntryObjects();
		if (deviceEntries != null) {
			for (int i = 0; i < deviceEntries.length; i++) {
				if (!pwmDesktop.getEntriesTable()
						.containsKey(deviceEntries[i].uuid)) {
					dirty = true;
					progress("Device->Desktop adding entry "
							+ deviceEntries[i].title + "\n");
					deviceEntries[i].add(pwmDesktop);
				}
			}
		}

		desktopEntries = pwmDesktop.getEntriesTable().sortedEntryObjects();
		deviceEntries = pwmDevice.getEntriesTable().sortedEntryObjects();

		if (desktopEntries == null)
			return (false);

		for (int i = 0; i < desktopEntries.length; i++) {
			PwEntry eDevice = pwmDevice.getEntriesTable().get(
					desktopEntries[i].uuid);

			if (desktopEntries[i].tLastAccess.getTime() / 5000 != eDevice.tLastAccess
					.getTime() / 5000) {
				long la = Math.max(desktopEntries[i].tLastAccess.getTime(),
						eDevice.tLastAccess.getTime());
				desktopEntries[i].tLastAccess = new Date(la);
				eDevice.tLastAccess = new Date(la);
				progress("Updated lastAccess for " + eDevice.title + "\n");
				dirty = true;
			}

			if (desktopEntries[i].tLastMod.getTime() / 5000 > eDevice.tLastMod
					.getTime() / 5000) {
				desktopEntries[i].copy(eDevice);
				progress("Updated Desktop->Device " + eDevice.title + "\n");
				dirty = true;
			}

			if (desktopEntries[i].tLastMod.getTime() / 5000 < eDevice.tLastMod
					.getTime() / 5000) {
				eDevice.copy(desktopEntries[i]);
				progress("Updated Device->Desktop " + eDevice.title + "\n");
				dirty = true;
			}
		}

		return (dirty);
	}

	private boolean reconcileGroups(PwManager pwmDesktop, PwManager pwmDevice) {
		boolean dirty = false;
		progress("Reconciling groups\n");

		PwGroup[] desktopGroups = pwmDesktop.getGroupsTable().groupObjects();
		Arrays.sort(desktopGroups, new Comparator() {
			public int compare(Object object, Object object1) {
				return (((PwGroup) object).level - ((PwGroup) object1).level);
			}
		});
		for (int i = 0; i < desktopGroups.length; i++) {
			if (!pwmDevice.getGroupsTable().containsKey(desktopGroups[i].groupId)) {
				dirty = true;
				progress("Desktop->Device adding group " + desktopGroups[i].name
						+ "\n");
				desktopGroups[i].add(pwmDevice);
			}
		}

		PwGroup[] deviceGroups = pwmDevice.getGroupsTable().groupObjects();
		Arrays.sort(deviceGroups, new Comparator() {
			public int compare(Object object, Object object1) {
				return (((PwGroup) object).level - ((PwGroup) object1).level);
			}
		});

		for (int i = 0; i < deviceGroups.length; i++) {
			if (!pwmDesktop.getGroupsTable().containsKey(deviceGroups[i].groupId)) {
				dirty = true;
				progress("Device->Desktop adding group " + deviceGroups[i].name
						+ "\n");
				deviceGroups[i].add(pwmDesktop);
			}
		}

		desktopGroups = pwmDesktop.getGroupsTable().sortedGroupObjects();
		deviceGroups = pwmDevice.getGroupsTable().sortedGroupObjects();

		for (int i = 0; i < desktopGroups.length; i++) {
			PwGroup gDevice = pwmDevice.getGroupsTable().get(
					desktopGroups[i].groupId);

			if (desktopGroups[i].tLastAccess.getTime() / 5000 != gDevice.tLastAccess
					.getTime() / 5000) {
				long la = Math.max(desktopGroups[i].tLastAccess.getTime(),
						gDevice.tLastAccess.getTime());
				desktopGroups[i].tLastAccess = new Date(la);
				gDevice.tLastAccess = new Date(la);
				progress("Updated lastAccess for " + gDevice.name + "\n");
				dirty = true;
			}
			if (desktopGroups[i].tLastMod.getTime() / 5000 > gDevice.tLastMod
					.getTime() / 5000) {
				desktopGroups[i].copy(gDevice);
				progress("Updated Desktop->Device " + gDevice.name + "\n");
				dirty = true;
			}

			if (desktopGroups[i].tLastMod.getTime() / 5000 < gDevice.tLastMod
					.getTime() / 5000) {
				gDevice.copy(desktopGroups[i]);
				progress("Updated Device->Desktop " + gDevice.name + "\n");
				dirty = true;
			}

		}
		return (dirty);
	}

	public void setSyncObjectDirty(SyncObject syncObject) {
		progress("setSyncObjectDirty "+syncObject.getUID());
	}

	public void clearSyncObjectDirty(SyncObject syncObject) {
		progress("clearSyncObjectDirty "+syncObject.getUID());
	}

	public int getSyncObjectCount() {
		progress("getSyncObjectCount");
		return 2;
	}

	public int getSyncVersion() {
		progress("getSyncVersion");
		return 2;
	}

	public String getSyncName() {
		return "KeePassBBDatabaseObject";
	}

	public String getSyncName(Locale locale) {
		return "KeePassBBDatabaseObject";
	}

	public SyncConverter getSyncConverter() {
		return this;
	}

	public void beginTransaction() {
		inTransaction = true;
		// sbProgress = new StringBuffer();
		psMessage.rc = 0;
		psMessage.message = "Ok";
		dboDX = null;
		progress("KeePassBB Sync Report");
		ApplicationManager appman = ApplicationManager.getApplicationManager();
		appman.postGlobalEvent(GUID_SYNCSTART);
		ApplicationDescriptor ad[] = appman.getVisibleApplications();
		boolean waitForApp = false;
		progress("Checking to see if app is running...");
		for (int i = 0; i < ad.length; i++)
			if (ad[i].getModuleName().toLowerCase().startsWith("keepassbb"))
				waitForApp = true;
		if (waitForApp) {
			progress("It is.");
			RuntimeStore rs = RuntimeStore.getRuntimeStore();
			String p = (String) rs.waitFor(GUID_SYNCPROCEED);
			rs.remove(GUID_SYNCPROCEED);
			if (!p.equalsIgnoreCase("OK")) {
				progress("App told Sync to not proceed: " + p);
				throw new RuntimeException("App told Sync to not proceed: " + p);
			}
			progress("Saved database before synching.");
		} else
			progress("It isn't.");
	}

	public void endTransaction() {
		inTransaction = false;
		Utils.logger("Ending transaction.");
		if (psMessage.rc == RC_CHANGES_MADE || psMessage.rc == RC_RESTORE || psMessage.rc == RC_LOADED_TO_DEVICE) {
			try {
				if (dboDX.filename == null || dboDX.filename.length() == 0) {
					dboDX.filename = Options.getStringOption(Options.OPTION_DEFAULTDBURL,
							PwSyncObject.generateDefaultFilename());
				}
				dboDX.save();
				progress("Database saved on device.");
			} catch (Exception e) {
				progress("Database error: " + e.getMessage());
				throw new RuntimeException(e.getMessage());
			} finally {
				dboDX = null;
			}
		}
		if (sbProgress != null)
			Utils.logger(sbProgress.toString());
		else
			Utils.logger("Transaction completed.");
		ApplicationManager appman = ApplicationManager.getApplicationManager();
		appman.postGlobalEvent(GUID_SYNCCOMPLETE, 0, 0, "OK", null);
	}

	public class PwSyncMessage implements SyncObject {
		public static final int UID = 120;
		public static final int FIELD_RC = 121;
		public static final int FIELD_MSG = 122;
		public int rc = 990;
		public String message = "Ok";

		public int getUID() {
			return UID;
		}

		public boolean convert(DataBuffer dataBuffer) {
			ConverterUtilities.writeInt(dataBuffer, FIELD_RC, rc);
			ConverterUtilities.writeString(dataBuffer, FIELD_MSG, message);
			return true;
		}

	}

	public class PwSyncCredentials implements SyncObject {
		public static final int UID = 130;
		public static final int FIELD_PASSWORD = 131;
		public static final int FIELD_KEYFILE = 132;
		public String password;
		public byte[] keyfile;
		
		public PwSyncCredentials(DataBuffer dataBuffer) throws IOException {
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			int chunk = 0;
			
			while(dataBuffer.available() > 0) {
				int type = ConverterUtilities.getType(dataBuffer);
				switch (type) {
				case FIELD_PASSWORD:
					byte[] pba = ConverterUtilities.readByteArray(dataBuffer, true);
					password = new String(pba);
					break;
				case FIELD_KEYFILE:
					byte[] ba = ConverterUtilities.readByteArray(dataBuffer, true);
					baos.write(ba);
					chunk++;
					break;
				default:
					ConverterUtilities.skipField(dataBuffer);
				}
			}
			keyfile = baos.toByteArray();
			if (keyfile.length == 0) keyfile = null;
			baos.close();
			
		}
		public int getUID() {
			return UID;
		}
	}

}
