package com.fairview5.keepassbb.kpobjects;

import java.io.ByteArrayOutputStream;
import java.io.IOException;

import net.rim.device.api.io.FileNotFoundException;
import net.rim.device.api.io.NoCopyByteArrayOutputStream;
import net.rim.device.api.synchronization.ConverterUtilities;
import net.rim.device.api.synchronization.SyncObject;
import net.rim.device.api.util.DataBuffer;

import com.fairview5.keepassbb.Utils;
import com.fairview5.keepassbb.file.KFile;

public class PwSyncObject implements SyncObject {
	public static final int UID = 100;
	public static final byte FIELD_TYPE_DATA = 101;
	public static final byte FIELD_TYPE_CHECKSUM = 102;
	public String filename;
	int dbLength;
	int dbFieldType;
	byte[] db;

	PwSyncObject(String filename) throws FileNotFoundException {
		this.filename = filename;
		if (filename == null || filename.length() == 0)
			throw new FileNotFoundException("No database found");
		KFile f = null;
		try {
			f = new KFile(filename);
			db = f.readContents();
			dbLength = db.length;
			this.dbFieldType = FIELD_TYPE_DATA;
		} catch (Throwable t) {
			throw new RuntimeException(t.getMessage());
		} finally {
			Utils.closeStream(f);
		}
	}

	PwSyncObject(byte[] db) {
		this.db = db;
		this.dbLength = db.length;
		this.dbFieldType = FIELD_TYPE_DATA;
	}
	
	PwSyncObject(DataBuffer dataBuffer) throws IOException {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		int chunk = 0;
		while(dataBuffer.available() > 0) {
			byte[] ba = ConverterUtilities.readByteArray(dataBuffer, true);
			baos.write(ba);
			chunk++;
		}
		this.db = baos.toByteArray();
		this.dbLength = baos.size();
		this.dbFieldType = FIELD_TYPE_DATA;
		baos.close();
	}
	
	PwSyncObject(NoCopyByteArrayOutputStream baos) {
		this.db = baos.getByteArray();
		this.dbLength = baos.size();
		this.dbFieldType = FIELD_TYPE_DATA;
	}
	
	public static String generateDefaultFilename() {
		String dir;
		if (Utils.getOSSoftwareVersion() <= 4.5) {
			dir = "/store/home/user/keepass/";
		} else {
			dir = "/system/keepass/";
		}
		return dir + "database.kdb";
	}
	
	public void save() {
		if (filename == null || filename.length() == 0) throw new RuntimeException("Filename can't be null");
		if (db == null)
			throw new RuntimeException("DB is null");
		KFile f = null;
		try {
			f = new KFile(filename);
			if (f.exists()) f.truncate();
			else f.create(true);
			f.write(db, 0, dbLength);
		} catch (Exception t) {
			t.printStackTrace();
			throw new RuntimeException("Write File: " + t.getMessage());
		} finally {
			Utils.closeStream(f);
		}

	}

	public boolean convert(DataBuffer dataBuffer) {
		int len = dbLength;
		if (len == 0) {
			ConverterUtilities.writeEmptyField(dataBuffer,
					PwSyncObject.FIELD_TYPE_DATA);
			return (true);
		}

		for (int i = 0; i < len;) {
			int csize = Math.min(8192, len - i);
			dataBuffer.writeShort(csize);
			dataBuffer.writeByte(dbFieldType);
			dataBuffer.write(db, i, csize);
			i += csize;
		}
		return true;
	}

	public int getUID() {
		return UID;
	}
}
