/*
KeePass for BlackBerry
Copyright 2007,2008 Fairview 5 Engineering, LLC <george.joseph@fairview5.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.fairview5.keepassbb.ui;

import java.util.Vector;

import net.rim.device.api.system.Characters;
import net.rim.device.api.system.Clipboard;
import net.rim.device.api.ui.*;
import net.rim.device.api.ui.component.*;
import net.rim.device.api.ui.container.MainScreen;
import net.rim.device.api.ui.container.VerticalFieldManager;

import com.fairview5.keepassbb.Options;
import com.fairview5.keepassbb.kpobjects.*;

public class SearchScreen extends MainScreen implements FieldChangeListener {

	public EditField ef = new EditField("Find: ", null);
	VerticalFieldManager vfm = new VerticalFieldManager(Manager.VERTICAL_SCROLL
			| Manager.VERTICAL_SCROLLBAR);
	ObjectListField lf = new ObjectListField();

	public SearchScreen() {
		super(Manager.NO_VERTICAL_SCROLL | Manager.NO_VERTICAL_SCROLLBAR);
		add(ef);
		ef.setChangeListener(this);
		add(new SeparatorField());
		vfm.add(lf);
		add(vfm);
	}

	public boolean isDirty() {
		return (false);
	}

	public boolean keyChar(char c, int status, int time) {
		if (c == Characters.ENTER || c == Characters.SPACE) {
			final Object o = lf.get(lf, lf.getSelectedIndex());
			if (o instanceof PwEntry) {
				int nix = lf.getSelectedIndex();
				openEntryMenuItem((PwEntry) o, nix, 0).run();
				return (true);
			}
		}
		return super.keyChar(c, status, time);
	}

	public void makeMenu(Menu menu, int instance) {

		int nix = lf.getSelectedIndex();
		if (nix >= 0) {
//			ContextMenu ctx = getContextMenu(instance);
			final Object o = lf.get(lf, nix);
			if ((o instanceof PwEntry)) {
				menu.add(openEntryMenuItem((PwEntry) o, nix, 1));
				menu.add(copyPasswordMenuItem((PwEntry) o, nix, 1));
			}
//			menu.add(ctx, true);
		}
		super.makeMenu(menu, Menu.INSTANCE_DEFAULT);
	}

	public boolean onMenu(int instance) {
		int nix = lf.getSelectedIndex();
		if (nix >= 0) {
			final Object o = lf.get(lf, nix);
			if ((o instanceof PwEntry && Options.getBooleanOption(
					Options.OPTION_SINGLECLICKENTRY, false))) {
				openEntryMenuItem((PwEntry) o, nix, 0).run();
				return (true);
			}
		}
		return (super.onMenu(instance));
	}

	public MenuItem openEntryMenuItem(final PwEntry o, final int nix, int pos) {
		return (new MenuItem("Open", pos, 0) {
			public void run() {
				PwEntry e = o;
				PwEntryScreen es = new PwEntryScreen(e);
				if (es.doModal() != Dialog.OK)
					return;
				try {
					if (PwManager.isDefaultDirty()
							&& Options.getIntOption(Options.OPTION_SAVETYPE,
									Options.SAVE_EVERYCHANGE) == Options.SAVE_EVERYCHANGE) {
						PwManager.pmCurrent.save();
					}
				} catch (Throwable e1) {
					Dialog.alert(e1.getMessage());
				}
			}
		});
	}

	public MenuItem copyPasswordMenuItem(final PwEntry o, final int nix, int pos) {
		return (new MenuItem("Copy Password", pos, 0) {
			public void run() {
				Clipboard.getClipboard().put(o.password);
			}
		});
	}

	public void fieldChanged(Field field, int context) {
		if (field != ef)
			return;
		String sc = ef.getText().toLowerCase();
		// if (sc == null || sc.length() == 0) lf.set(new Object[] {});

		PwEntry[] pwe = PwManager.pmCurrent.getEntriesTable()
				.sortedEntryObjects();
		Vector v = new Vector();
		int st = Options.getIntOption(Options.OPTION_SEARCHTYPE,
				Options.SEARCH_TITLESTARTSWITH);
		for (int i = 0; i < pwe.length; i++) {
			if (st == Options.SEARCH_TITLESTARTSWITH) {
				if (pwe[i].title.toLowerCase().startsWith(sc)) {
					v.addElement(pwe[i]);
				}
			} else {
				if (pwe[i].title.toLowerCase().indexOf(sc) > -1) {
					v.addElement(pwe[i]);
					continue;
				}
				if (pwe[i].url.toLowerCase().indexOf(sc) > -1) {
					v.addElement(pwe[i]);
					continue;
				}
				if (pwe[i].username.toLowerCase().indexOf(sc) > -1) {
					v.addElement(pwe[i]);
					continue;
				}
				if (pwe[i].password.toLowerCase().indexOf(sc) > -1) {
					v.addElement(pwe[i]);
					continue;
				}
				if (pwe[i].additional.toLowerCase().indexOf(sc) > -1) {
					v.addElement(pwe[i]);
					continue;
				}
			}
		}
		Object[] a = new Object[v.size()];
		v.copyInto(a);
		lf.set(a);

	}
}
