/*
KeePass for BlackBerry
Copyright 2007 Fairview 5 Engineering, LLC <george.joseph@fairview5.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.fairview5.keepassbb.ui;

import net.rim.device.api.system.*;
import net.rim.device.api.ui.*;
import net.rim.device.api.ui.component.*;
import net.rim.device.api.util.CharacterUtilities;

import com.fairview5.keepassbb.KeePassBBScreen;
import com.fairview5.keepassbb.Options;
import com.fairview5.keepassbb.kpobjects.*;

public final class TreeScreen extends TreeField {
	private KeePassBBScreen ks;
	private TreeField tf;
	public static int NUM_ICONS = 69;
	public static Bitmap mIcon[] = new Bitmap[NUM_ICONS];

	static {
		int i;
		try {
			for (i = 0; i < NUM_ICONS; i++) {
				mIcon[i] = EncodedImage.getEncodedImageResource(i + "_gt.png").getBitmap();
			}
		} catch (Exception e) {
		}
	}

	public TreeScreen(KeePassBBScreen ks) {
		super(new TreeFieldCallback() {
			public void drawTreeItem(TreeField tf, Graphics graphics, int node, int y, int width, int indent) {
				Object cookie = tf.getCookie(node);
				Bitmap ic = null;
				int ix = 0;
				if (cookie instanceof PwEntry) {
					ix = ((PwEntry) cookie).imageId;
				} else if (cookie instanceof PwGroup) {
					ix = ((PwGroup) cookie).imageId;
				}
				if (ix >= 0 && ix < TreeScreen.NUM_ICONS)
					ic = TreeScreen.mIcon[ix];
				if (ic != null && !Options.getBooleanOption(Options.OPTION_HIDEICONS, false)) {
					graphics.drawBitmap(indent, y, ic.getWidth(), ic.getHeight(), ic, 0, 0);
					indent += ic.getWidth();
				}
				graphics.drawText(cookie.toString(), indent, y, Graphics.ELLIPSIS, width);
			}
		}, Field.FOCUSABLE);
		this.ks = ks;
		tf = this;
		setEmptyString("No database loaded", DrawStyle.HCENTER);
	}

	public void loadTree(PwManager pwm) {
		traverseTree(pwm.rootGroup, 0);
	}

	public void traverseTree(PwGroup group, int parentNode) {
		PwGroup[] cg = group.childGroups.sortedGroupObjects();
		int n = -1;
		if (cg != null)
			for (int i = 0; i < cg.length; i++) {
				if (n < 0)
					n = tf.addChildNode(parentNode, cg[i]);
				else
					n = tf.addSiblingNode(n, cg[i]);
				traverseTree(cg[i], n);
			}

		PwEntry[] ce = group.childEntries.sortedEntryObjects();
		n = -1;
		if (ce != null)
			for (int i = 0; i < ce.length; i++) {
				if (n < 0)
					n = tf.addChildNode(parentNode, ce[i]);
				else
					n = tf.addSiblingNode(n, ce[i]);
			}
	}

	public boolean keyChar(char c, int status, int time) {
		int nix = tf.getCurrentNode();
		if (nix < 0)
			return (false);
		Object o = tf.getCookie(nix);

		if (c == Characters.ENTER) {
			if (o instanceof PwEntry) {
				miOpenEntry.run();
				return (true);
			}
			if (o instanceof PwGroup) {
				miOpenGroup.run();
				return (true);
			}
		}
		if (Options.getBooleanOption(Options.OPTION_SEARCHFYAT, true) && tf.getNodeCount() > 0 && status == 0
				&& (CharacterUtilities.isLetter(c) || CharacterUtilities.isDigit(c))) {
			SearchScreen sc = new SearchScreen();
			sc.ef.insert(c + "");
			sc.ef.setCursorPosition(1);
			UiApplication.getUiApplication().pushModalScreen(sc);
		}
		return super.keyChar(c, status, time);
	}

	void addMI(Menu menu, MenuItem mi, int ordinal, int priority) {
		mi.setOrdinal(ordinal);
		mi.setPriority(priority);
		menu.add(mi);
	}

	public void makeMenu(Menu ctx) {
		final int nix = tf.getCurrentNode();

		if (nix >= 0) {
			final Object o = tf.getCookie(tf.getCurrentNode());
			if (o instanceof PwEntry) {
				addMI(ctx,miOpenEntry, 100, 0);
				addMI(ctx,miCopyPassword, 110, 0);
				if (!PwManager.isDefaultLegacy() && !Options.isReadonly()) {
					addMI(ctx,miDeleteEntry, 120, 0);
				}
			}
			if (o instanceof PwGroup) {

				if (!PwManager.isDefaultLegacy() && !Options.isReadonly()) {
					addMI(ctx,miNewEntry, 100, 0);
				}
				ctx.add(MenuItem.separator(120));
				if (tf.getExpanded(nix)) {
					addMI(ctx,miCollapseGroup, 130, 0);
				} else {
					addMI(ctx,miExpandGroup, 140, 0);
				}
				if (!PwManager.isDefaultLegacy() && !Options.isReadonly()) {
					addMI(ctx,miOpenGroup, 150, 0);
					addMI(ctx,miDeleteGroup, 160, 0);
					addMI(ctx,miNewSubGroup, 170, 0);
					addMI(ctx,miNewTopGroup, 180, 0);
				}
			}
		}
	}

	protected boolean navigationClick(int status, int time) {
		final int nix = tf.getCurrentNode();
		if (nix >= 0 && Trackball.isSupported()) {
			Object o = tf.getCookie(nix);
			if ((o instanceof PwEntry)) {
				miOpenEntry.run();
				return (true);
			}

			if ((o instanceof PwGroup)) {
				tf.setExpanded(nix, !tf.getExpanded(nix));
				return (true);
			}
		}
		return (false);
	}

	private void cleanup() {
		PwManager.setDefaultDirty(true);
		ks.setTitle();
		if (Options.getIntOption(Options.OPTION_SAVETYPE, Options.SAVE_EVERYCHANGE) == Options.SAVE_EVERYCHANGE) {
			ks.save();
		}
	}

	MenuItem miSearch = new MenuItem("Find", 0, 0) {
		public void run() {
			SearchScreen sc = new SearchScreen();
			UiApplication.getUiApplication().pushModalScreen(sc);
		}
	};

	MenuItem miDeleteEntry = new MenuItem("Delete Entry", 0, 0) {
		public void run() {
			int nix = tf.getCurrentNode();
			final PwEntry o = (PwEntry) tf.getCookie(nix);

			if (Options.getBooleanOption(Options.OPTION_DELETEPROMPT, true)) {
				int rc = Dialog.ask(Dialog.D_DELETE, "Delete entry "
						+ o.title);
				if (rc != Dialog.DELETE)
					return;
			}
			(o).delete(PwManager.pmCurrent);
			tf.deleteSubtree(nix);
			cleanup();
		}
	};

	MenuItem miOpenEntry = new MenuItem("Open Entry", 0, 0) {
		public void run() {
			int nix = tf.getCurrentNode();
			final PwEntry o = (PwEntry) tf.getCookie(nix);
			PwEntryScreen es = new PwEntryScreen(o);
			es.show();
		}
	};
	MenuItem miCopyPassword = new MenuItem("Copy Password", 0, 0) {
		public void run() {
			int nix = tf.getCurrentNode();
			final PwEntry o = (PwEntry) tf.getCookie(nix);
			Clipboard.getClipboard().put(o.password);
		}
	};

	MenuItem miOpenGroup = new MenuItem("Open Group", 0, 0) {
		public void run() {
			int nix = tf.getCurrentNode();
			final PwGroup o = (PwGroup) tf.getCookie(nix);
			PwGroupScreen es = new PwGroupScreen(o);
			es.show();
		}
	};

	MenuItem miCollapseGroup = new MenuItem("Collapse Group", 0, 0) {
		public void run() {
			int nix = tf.getCurrentNode();
			tf.setExpanded(nix, false);
		}
	};

	MenuItem miExpandGroup = new MenuItem("Expand Group", 0, 0) {
		public void run() {
			int nix = tf.getCurrentNode();
			tf.setExpanded(nix, true);
		}
	};

	MenuItem miDeleteGroup = new MenuItem("Delete Group", 0, 0) {
		public void run() {
			int nix = tf.getCurrentNode();
			final PwGroup o = (PwGroup) tf.getCookie(nix);
			if (Options.getBooleanOption(Options.OPTION_DELETEPROMPT, true)) {
				int rc = Dialog.ask(Dialog.D_DELETE, "Delete group "
						+ o.name);
				if (rc != Dialog.DELETE)
					return;
			}

			(o).delete(PwManager.pmCurrent);
			tf.deleteSubtree(nix);
			cleanup();
		}
	};

	MenuItem miNewEntry = new MenuItem("Add Entry", 0, 0) {
		public void run() {
			try {
				int nix = tf.getCurrentNode();
				final PwGroup o = (PwGroup) tf.getCookie(nix);
				PwEntry pwe = new PwEntry(o);
				pwe.add(PwManager.pmCurrent);
				PwEntryScreen pes = new PwEntryScreen(pwe);
				int rc = pes.doModal();
				if (rc != Dialog.OK) {
					pwe.delete(PwManager.pmCurrent);
					return;
				}
				// cleanup();
				tf.addChildNode(nix, pwe);
			} catch (Throwable e) {
				Dialog.alert(e.getMessage());
			}
		}
	};

	MenuItem miNewSubGroup = new MenuItem("Add SubGroup", 0, 0) {
		public void run() {
			try {
				int nix = tf.getCurrentNode();
				final PwGroup ppwg = (PwGroup) tf.getCookie(nix);
				PwGroup pwg = new PwGroup(ppwg, null);
				PwGroupScreen pgs = new PwGroupScreen(pwg);
				int rc = pgs.doModal();
				if (rc != Dialog.OK) {
					pwg.delete(PwManager.pmCurrent);
					return;
				}

				pwg.add(PwManager.pmCurrent);

				cleanup();
				tf.addChildNode(nix, pwg);
			} catch (Throwable e) {
				Dialog.alert(e.getMessage());
			}
		}
	};

	MenuItem miNewTopGroup = new MenuItem("Add Group", 0, 0) {
		public void run() {
			int nix = tf.getCurrentNode();
			PwGroup pwg = new PwGroup(PwManager.pmCurrent.rootGroup, null, 48, 0);
			PwGroupScreen pgs = new PwGroupScreen(pwg);
			int rc = pgs.doModal();
			if (rc != Dialog.OK) {
				pwg.delete(PwManager.pmCurrent);
				return;
			}
			PwManager.pmCurrent.getGroupsTable().put(pwg.groupId, pwg);
			PwManager.pmCurrent.rootGroup.childGroups.put(pwg.groupId, pwg);
			cleanup();
			tf.addChildNode(nix, pwg);
		}
	};
}
